#!/usr/bin/wish -f

# call: plot.tcl ../Data/square.ccm ../

# ChangeLog:
#  May08 JS: replace tempString by tmpS and handle multiple comments in .ccm

set Font1 -*-Helvetica-Bold-R-Normal--*-300-*-*-*-*-*-*
set Font7 -*-Helvetica-Medium-R-Normal--*-140-*-*-*-*-*-*
set Font2 -*-Helvetica-Medium-R-Normal--*-100-*-*-*-*-*-*
set Font3 -Adobe-Courier-Bold-R-Normal--*-150-*-*-*-*-*-*
set Font4 -Adobe-Courier-Bold-R-Normal--*-180-*-*-*-*-*-*
set Font5 -Adobe-Courier-Bold-R-Normal--*-180-*-*-*-*-*-*
set Font6 -Adobe-Courier-Bold-R-Normal--*-120-*-*-*-*-*-*

set    script [lindex $argv 0]
set    path   [lindex $argv 1]

append path "Data/"

set fnameIn $path
append fnameIn $script

set fileid [open $fnameIn r]

set colour 1

# get control parameters first from script.ccm file

proc readNextLine {fileid} {
# skip comments
  set tmpS "//"
  while {[string compare -length 2 "//" $tmpS] == 0} {
    gets $fileid tmpS
    puts "Read in: $tmpS"
  }
  return $tmpS
}

puts "Reading from $fnameIn"
set tmpS [readNextLine $fileid]    ;# "output level=" ignored
set tmpS [readNextLine $fileid]    ;# "find ... expectation values=" ignored
set tmpS [readNextLine $fileid]
set dummy [scan $tmpS "work directory=%20s" tempPath]
if { $dummy == 0 } {
    puts "Error in scanning work directory name"
    exit
}
set tmpS [readNextLine $fileid]
set dummy [scan $tmpS "output file=%20s" filename]
if { $dummy == 0 } {
    puts "Error in scanning output file name"
    exit
}
#   solve.C parameters and plot.tcl parameters

set maxxtxt "1.0"
set minxtxt "-1.0"
set incxtxt "0.2"

set maxytxt "1.0"
set minytxt "-1.0"
set incytxt "0.2"

set tmpS [readNextLine $fileid]
set dummy [scan $tmpS "minimum=%10s" minxtxt]
set tmpS [readNextLine $fileid]
set dummy [scan $tmpS "maximum=%10s" maxxtxt]
set tmpS [readNextLine $fileid]
set dummy [scan $tmpS "increment=%16s" incxtxt]
set tmpS [readNextLine $fileid]
set dummy [scan $tmpS "minimumy=%10s" minytxt]
set tmpS [readNextLine $fileid]
set dummy [scan $tmpS "maximumy=%10s" maxytxt]

puts "stop reading"
close $fileid

append filename ".output"

set maxx [format "%f" $maxxtxt]
set minx [format "%f" $minxtxt]
set miny [format "%f" $minytxt]
set maxy [format "%f" $maxytxt]
set incx [format "%f" $incxtxt]
set incry [expr abs($maxy - $miny)/10.0 ]

set count 0
set output_variable 1

label .lw -text "Plot Results" -font $Font1 -width 15\
	    -foreground blue -height 2
pack .lw -anchor n -pady 2

frame .params -bd 4
pack .params -expand yes -fill x 

label .params.l1 -font $Font3 -text "Max. x:" 
entry .params.e1 -relief sunken -width 15 -textvariable maxxtxt 
label .params.l2 -font $Font3 -text "Min. x:" 
entry .params.e2 -relief sunken -width 15 -textvariable minxtxt
label .params.l3 -font $Font3 -text "Max. y:" 
entry .params.e3 -relief sunken -width 15 -textvariable maxytxt 
label .params.l4 -font $Font3 -text "Min. y:" 
entry .params.e4 -relief sunken -width 15 -textvariable minytxt

label .params.l5 -font $Font3 -text "File:" 
entry .params.e5 -relief sunken -width 15 -textvariable filename 
label .params.l6 -font $Font3 -text "Path:" 
entry .params.e6 -relief sunken -width 15 -textvariable tempPath

menubutton .params.output -text "Plot What?:"  -background blue  -width 11 \
	-foreground white  -font $Font3 -padx 1m -relief groove\
	-pady 2 -menu   .params.output.menu

menu .params.output.menu  -background blue  -foreground white
.params.output.menu add radiobutton -label "E_g/N" -font $Font6 \
	-variable  output_variable -value 1
.params.output.menu add radiobutton -label "Magnetisation" -font $Font6 \
	-variable  output_variable -value 2
.params.output.menu add radiobutton -label "d^2 (E_g/N)/d x^2" \
	-font $Font6 -variable  output_variable -value 3
.params.output.menu add radiobutton -label "x_1" -font $Font6 \
	-variable  output_variable -value 4

grid config .params.l1 -column 0 -row 0 -sticky e -pady 2 -padx 2 
grid config .params.e1 -column 1 -row 0 -sticky w -pady 2 -padx 2 
grid config .params.l2 -column 2 -row 0 -sticky e -pady 2 -padx 2 
grid config .params.e2 -column 3 -row 0 -sticky w -pady 2 -padx 2 
grid config .params.l3 -column 4 -row 0 -sticky e -pady 2 -padx 2 
grid config .params.e3 -column 5 -row 0 -sticky w -pady 2 -padx 2 
grid config .params.l4 -column 6 -row 0 -sticky e -pady 2 -padx 2 
grid config .params.e4 -column 7 -row 0 -sticky w -pady 2 -padx 2 

grid config .params.l5 -column 0 -row 1 -sticky e -pady 2 -padx 2 
grid config .params.e5 -column 1 -row 1 -sticky w -pady 2 -padx 2 
grid config .params.l6 -column 2 -row 1 -sticky e -pady 2 -padx 2 
grid config .params.e6 -column 3 -row 1 -sticky w -pady 2 -padx 2 

grid config .params.output -column 5 -row 1 -sticky e -pady 2 -padx 2 

canvas .c -height 600 -width 600 -background white -highlightbackground black
pack .c -side top -padx 5 -pady 5

# note that create line min(x) min(y) max(x) max(y) 

set aa [ expr abs($maxy - $miny)]
if {$maxy < 0} {
    set xaxis 30
} else {
    if {$miny < 0} {
	set xaxis  [ expr 510*$maxy/$aa + 30]
    } else {
	set xaxis 540
    }
} 
.c create line 60 $xaxis 570 $xaxis -tags "line1" -fill grey 

#y axis

set bb [ expr abs($maxx - $minx)]
if {$minx > 0} {
    set yaxis  60 
} else {
    if {$maxx > 0} {
	set yaxis  [ expr -510*$minx/$bb + 60]
    } else {
	set yaxis 570
    }
}
.c create line $yaxis 30 $yaxis 540 -tags "line2" -fill grey 

#also do axes round the edge 

.c create line 60 30 60 540 
.c create line 570 30 570 540 
.c create line 60 30 570 30 
.c create line 60 540 570 540 


set c 0
while {$c <= 10} {
    set xpos [expr 60+51*$c]
    set ypos [expr 30+51*$c]
    set yval [expr $maxy + $c*($miny - $maxy)/10.0]
    set xval [expr $minx + $c*($maxx - $minx)/10.0]
    .c create line $xpos 535 $xpos 540 
    .c create text $xpos 560 -text [format "%5.3f" $xval]
    .c create line $xpos 30 $xpos 35 
    .c create text 30 $ypos -text [format "%7.4f" $yval]
    .c create line 60 $ypos 65 $ypos 
    .c create line 565 $ypos 570 $ypos 
    set c [expr $c + 1] 
}

if { $output_variable == 1 } {
    .c create text 65 15 -font Font3 -text "E_g/N"
} elseif { $output_variable == 2 } {
    .c create text 65 15 -font Font3 -text "M"
}  elseif { $output_variable == 3 } {
    .c create text 65 15 -font Font3 -text "d^2 (E_g/N)/d x^2"
}  elseif { $output_variable == 4 } {
    .c create text 65 15 -font Font3 -text "X(1)"
} 

.c create text 310 585 -font Font3 -text "X"

frame .mbar4 -bd 4
pack .mbar4 -expand yes -fill x 
button .mbar4.b1 -text "Exit" -font $Font4 -height 1 -width 20 \
	-command { exit }

button .mbar4.b2 -text "Clear/Rescale" -font $Font4 -height 1 -width 20 \
	-command { 
    rescale_clear
    refresh_buttons
}

button .mbar4.b3 -text "Plot" -font $Font4 -height 1 -width 20 \
	-command { 
    plot_data
}

pack .mbar4.b1 .mbar4.b2 .mbar4.b3 -side left -fill x -padx 5 

label .l3 -text "@COPYRIGHT DAMIAN FARNELL 2002" \
     -height 1 -foreground blue -font $Font7
# Copyright at the bottom.

pack .l3 -anchor n -pady 5

proc rescale_clear { } {

    global Font1 
    global Font2 
    global Font3 
    global Font4 
    global Font5 
    global Font6 
    global Font7 

    global maxx
    global maxy
    global minx
    global miny
    global maxxtxt
    global maxytxt
    global minxtxt
    global minytxt

    global yaxis
    global xaxis

    global output_variable
    global colour

    set colour 1

    #first delete the text scale key

    set maxx [format "%f" $maxxtxt]
    set minx [format "%f" $minxtxt]
    set miny [format "%f" $minytxt]
    set maxy [format "%f" $maxytxt]
    
    destroy .c
    
    canvas .c -height 600 -width 600 -background white \
	    -highlightbackground black
    pack .c -side top -padx 5 -pady 5
    
    # note that create line min(x) min(y) max(x) max(y) 
    
    set aa [ expr abs($maxy - $miny)]
    if {$maxy < 0} {
	set xaxis 30
    } else {
	if {$miny < 0} {
	    set xaxis  [ expr 510*$maxy/$aa + 30]
	} else {
	    set xaxis 540
	}
    } 
    .c create line 60 $xaxis 570 $xaxis -tags "line1" -fill grey 
    
    #y axis
    
    set bb [ expr abs($maxx - $minx)]
    if {$minx > 0} {
	set yaxis  60 
    } else {
	if {$maxx > 0} {
	    set yaxis  [ expr -510*$minx/$bb + 60]
	} else {
	    set yaxis 570
	}
    }
    .c create line $yaxis 30 $yaxis 540 -tags "line2" -fill grey 
    
    #also do axes round the edge 
    
    .c create line 60 30 60 540 
    .c create line 570 30 570 540 
    .c create line 60 30 570 30 
    .c create line 60 540 570 540 
    
    
    set c 0
    while {$c <= 10} {
	set xpos [expr 60+51*$c]
	set ypos [expr 30+51*$c]
	set yval [expr $maxy + $c*($miny - $maxy)/10.0]
	set xval [expr $minx + $c*($maxx - $minx)/10.0]
	.c create line $xpos 535 $xpos 540 
	.c create text $xpos 560 -text [format "%5.3f" $xval]
	.c create line $xpos 30 $xpos 35 
	.c create text 30 $ypos -text [format "%7.4f" $yval]
	.c create line 60 $ypos 65 $ypos 
	.c create line 565 $ypos 570 $ypos 
	set c [expr $c + 1] 
    }
    
    if { $output_variable == 1 } {
	.c create text 65 15 -font Font3 -text "E_g/N"
    } elseif { $output_variable == 2 } {
	.c create text 65 15 -font Font3 -text "M"
    }  elseif { $output_variable == 3 } {
	.c create text 65 15 -font Font3 -text "d^2 (E_g/N)/d x^2"
    }  elseif { $output_variable == 4 } {
	.c create text 65 15 -font Font3 -text "X(1)"
    } 
    
    .c create text 310 585 -font Font3 -text "X"

    return
    
}

proc refresh_buttons { } {

    global Font1 
    global Font2 
    global Font3 
    global Font4 
    global Font5 
    global Font6 
    global Font7 

    destroy .mbar4
    destroy .l3

    frame .mbar4 -bd 4
    pack .mbar4 -expand yes -fill x 
    button .mbar4.b1 -text "Exit" -font $Font4 -height 1 -width 20 \
	    -command { exit }
    
    button .mbar4.b2 -text "Clear/Rescale" -font $Font4 -height 1 -width 20 \
	    -command { 
	rescale_clear
	refresh_buttons
    }
    
    button .mbar4.b3 -text "Plot" -font $Font4 -height 1 -width 20 \
	    -command { 
	plot_data
    }
    
    pack .mbar4.b1 .mbar4.b2 .mbar4.b3 -side left -fill x -padx 5 
    
    label .l3 -text "@COPYRIGHT DAMIAN FARNELL 2002" \
	    -height 1 -foreground blue -font $Font7
    # Copyright at the bottom.
    
    pack .l3 -anchor n -pady 5

}

proc plot_data { } {

    global Font1 
    global Font2 
    global Font3 
    global Font4 
    global Font5 
    global Font6 
    global Font7 

    global maxx
    global maxy
    global minx
    global miny

    global path
    global tempPath
    global filename
    global output_variable
    global xaxis
    global yaxis
    global colour

    set count 0
    set flag 0
    
    set fname $tempPath
    append fname $filename
    set fileid [open $fname r] 
    
    while { $count < 200000 } {
	
	gets $fileid line 
	
	if { $line == "" } {
	    break
	}
	scan $line "%f %f %f %f %f" x ty(1) ty(2) ty(3) ty(4) 
	
	set dummy1 [format "set y \$ty(%d)" $output_variable]
	eval $dummy1
	
	set aa [ expr abs($maxx - $minx)]
	set bb [ expr abs($maxy - $miny)]
	
	if {$minx > 0} {
	    set x [ expr 510*($x-$minx)/$aa + $yaxis]
	} else {
	    if {$maxx < 0} {
		set x [ expr 510*abs($x-$maxx)/$aa + $yaxis]
	    } else {
		set x [ expr 510*$x/$aa + $yaxis]
	    }
	}
	
	if {$maxy < 0} {
	    set y [ expr 510*($maxy-$y)/$bb + 30]
	} else {
	    if {$miny < 0} {
		set y [ expr -510*$y/$bb + $xaxis]
	    } else {
		set y [ expr 510*($maxy-$y)/$bb + 30]
	    }
	} 

	if { $x < 60 } {
	    set flag 0
	}
	if { $x > 570 } {
	   set flag 0
	} 

	if { $y < 30 } {
	    set flag 0
	}
	if { $y > 540 } {
	   set flag 0
	} 
	
	if {$flag == 0} {
	    switch $colour {
		1 {.c create line $x $y $x $y -tags "l$count" -width 2 \
			-fill black }
		2 {.c create line $x $y $x $y -tags "l$count" -width 2 \
			-fill red }
		3 {.c create line $x $y $x $y -tags "l$count" -width 2 \
			-fill blue }
		4 {.c create line $x $y $x $y -tags "l$count" -width 2 \
			-fill green }
		5 {.c create line $x $y $x $y -tags "l$count" -width 2 \
			-fill orange }
		6 {.c create line $x $y $x $y -tags "l$count" -width 2 \
			-fill grey }
		7 {.c create line $x $y $x $y -tags "l$count" -width 2 \
			-fill violet}
		8 {.c create line $x $y $x $y -tags "l$count" -width 2 \
			-fill brown}
		9 {.c create line $x $y $x $y -tags "l$count" -width 2 \
			-fill yellow}
	    }
	} else {
	    switch $colour {
		1 {.c create line $ox $oy $x $y -tags "l$count" -width 2 \
			-fill black }
		2 {.c create line $ox $oy $x $y -tags "l$count" -width 2 \
			-fill red }
		3 {.c create line $ox $oy $x $y -tags "l$count" -width 2 \
			-fill blue }
		4 {.c create line $ox $oy $x $y -tags "l$count" -width 2 \
			-fill green }
		5 {.c create line $ox $oy $x $y -tags "l$count" -width 2 \
			-fill orange }
		6 {.c create line $ox $oy $x $y -tags "l$count" -width 2 \
			-fill grey }
		7 {.c create line $ox $oy $x $y -tags "l$count" -width 2 \
			-fill violet}
		8 {.c create line $ox $oy $x $y -tags "l$count" -width 2 \
			-fill brown}
		9 {.c create line $ox $oy $x $y -tags "l$count" -width 2 \
			-fill yellow}
	    }
	}
	
	set ox $x
	set oy $y
	set flag 1
	
	set count [expr $count + 1]
    }
        
    close $fileid

    set colour [ expr $colour + 1]

    if { $colour > 9 } {
	set colour 1
    }

    return

}